<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 496 $
 * $Id: AccrualPolicyFactory.class.php 496 2007-01-04 20:21:26Z ipso $
 * $Date: 2007-01-04 12:21:26 -0800 (Thu, 04 Jan 2007) $
 */
class AccrualPolicyFactory extends Factory {
	protected $table = 'accrual_policy';
	protected $pk_sequence_name = 'accrual_policy_id_seq'; //PK Sequence name

	protected $company_obj = NULL;

	protected $type_options = array(
										10 => 'Standard',
										20 => 'Calendar Based',
//										30 => 'Hours Based',
									);
									
	protected $apply_frequency_options = array(
										10 => 'each Pay Period',
										20 => 'Yearly',
										30 => 'Monthly',
										40 => 'Weekly',
									);

	protected $company_validator_msg = 'Company is invalid';
	protected $company_validator_label = 'company';

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $name_validator_unique_msg = 'Name is already in use';
	protected $name_validator_islength_msg = 'Name is invalid';
	protected $name_validator_label = 'name';

	protected $inuse_validator_msg = 'This accrual policy is in use';
	protected $inuse_validator_label = 'in_use';

	protected $minimumtime_validator_isnumeric_msg = 'Incorrect Minimum Time';
	protected $minimumtime_validator_label = 'minimum_time';

	protected $maximumtime_validator_isnumeric_msg = 'Incorrect Maximum Time';
	protected $maximumtime_validator_label = 'maximum_time';

	protected $applyfrequency_validator_options_msg = 'Incorrect frequency';
	protected $applyfrequency_validator_label = 'apply_frequency';

	protected $applyfrequencymonth_validator_options_msg = 'Incorrect frequency month';
	protected $applyfrequencymonth_validator_label = 'apply_frequency_month';

	protected $applyfrequencydayofmonth_validator_options_msg = 'Incorrect frequency day of month';
	protected $applyfrequencydayofmonth_validator_label = 'apply_frequency_day_of_month';

	protected $applyfrequencydayofweek_validator_options_msg = 'Incorrect frequency day of week';
	protected $applyfrequencydayofweek_validator_label = 'apply_frequency_day_of_week';

	protected $milestonerollovermonth_validator_options_msg = 'Incorrect milestone rollover month';
	protected $milestonerollovermonth_validator_label = 'milestone_rollover_month';

	protected $milestonerolloverdayofmonth_validator_options_msg = 'Incorrect milestone rollover day of month';
	protected $milestonerolloverdayofmonth_validator_label = 'milestone_rollover_day_of_month';

	protected $minimumemployeddays_validator_isnumeric_msg = 'Incorrect Minimum Employed days';
	protected $minimumemployeddays_validator_label = 'minimum_employed_days';

	function getCompanyObject() {
		if ( is_object($this->company_obj) ) {
			return $this->company_obj;
		} else {
			$clf = new CompanyListFactory();
			$this->company_obj = $clf->getById( $this->getCompany() )->getCurrent();

			return $this->company_obj;
		}
	}

	function getCompany() {
		if ( isset($this->data['company_id']) ) {
			return $this->data['company_id'];
		}

		return FALSE;
	}
	function setCompany($id) {
		$id = trim($id);

		Debug::Text('Company ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$clf = new CompanyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->company_validator_label,
													$clf->getByID($id),
													$this->company_validator_msg
													) ) {

			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getType() {
		if ( isset($this->data['type_id']) ) {
			return $this->data['type_id'];
		}

		return FALSE;
	}
	function setType($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('type') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$value,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getName() {
		if ( isset($this->data['name']) ) {
			return $this->data['name'];
		}

		return FALSE;
	}
	function setName($name) {
		$name = trim($name);
		if (	$this->Validator->isLength(	$this->name_validator_label,
											$name,
											$this->name_validator_islength_msg,
											2,50)
						) {

			$this->data['name'] = $name;

			return TRUE;
		}

		return FALSE;
	}

	function getMinimumTime() {
		if ( isset($this->data['minimum_time']) ) {
			return (int)$this->data['minimum_time'];
		}

		return FALSE;
	}
	function setMinimumTime($int) {
		$int = trim($int);

		if  ( empty($int) ){
			$int = 0;
		}

		if 	(	$this->Validator->isNumeric(		$this->minimumtime_validator_label,
													$int,
													$this->minimumtime_validator_isnumeric_msg) ) {
			$this->data['minimum_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getMaximumTime() {
		if ( isset($this->data['maximum_time']) ) {
			return (int)$this->data['maximum_time'];
		}

		return FALSE;
	}
	function setMaximumTime($int) {
		$int = trim($int);

		if  ( empty($int) ){
			$int = 0;
		}

		if 	(	$this->Validator->isNumeric(		$this->maximumtime_validator_label,
													$int,
													$this->maximumtime_validator_isnumeric_msg) ) {
			$this->data['maximum_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	//
	// Calendar
	//
	function getApplyFrequency() {
		if ( isset($this->data['apply_frequency_id']) ) {
			return $this->data['apply_frequency_id'];
		}

		return FALSE;
	}
	function setApplyFrequency($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('apply_frequency') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->applyfrequency_validator_label,
											$value,
											$this->applyfrequency_validator_options_msg,
											$this->getOptions('apply_frequency')) ) {

			$this->data['apply_frequency_id'] = $value;

			return TRUE;
		}

		return FALSE;
	}
	
	function getApplyFrequencyMonth() {
		if ( isset($this->data['apply_frequency_month']) ) {
			return $this->data['apply_frequency_month'];
		}

		return FALSE;
	}
	function setApplyFrequencyMonth($value) {
		$value = trim($value);

		$key = Option::getByValue($value, TTDate::getMonthOfYearArray() );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->applyfrequencymonth_validator_label,
											$value,
											$this->applyfrequencymonth_validator_options_msg,
											TTDate::getMonthOfYearArray() ) ) {

			$this->data['apply_frequency_month'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getApplyFrequencyDayOfMonth() {
		if ( isset($this->data['apply_frequency_day_of_month']) ) {
			return $this->data['apply_frequency_day_of_month'];
		}

		return FALSE;
	}
	function setApplyFrequencyDayOfMonth($value) {
		$value = trim($value);

		$key = Option::getByValue($value, TTDate::getDayOfMonthArray() );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->applyfrequencydayofmonth_validator_label,
											$value,
											$this->applyfrequencydayofmonth_validator_options_msg,
											TTDate::getDayOfMonthArray() ) ) {

			$this->data['apply_frequency_day_of_month'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getApplyFrequencyDayOfWeek() {
		if ( isset($this->data['apply_frequency_day_of_week']) ) {
			return $this->data['apply_frequency_day_of_week'];
		}

		return FALSE;
	}
	function setApplyFrequencyDayOfWeek($value) {
		$value = trim($value);

		$key = Option::getByValue($value, TTDate::getDayOfWeekArray() );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->applyfrequencydayofweek_validator_label,
											$value,
											$this->applyfrequencydayofweek_validator_options_msg,
											TTDate::getDayOfWeekArray() ) ) {

			$this->data['apply_frequency_day_of_week'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getMilestoneRolloverHireDate() {
		return $this->fromBool( $this->data['milestone_rollover_hire_date'] );
	}
	function setMilestoneRolloverHireDate($bool) {
		$this->data['milestone_rollover_hire_date'] = $this->toBool($bool);

		return true;
	}

	function getMilestoneRolloverMonth() {
		if ( isset($this->data['milestone_rollover_month']) ) {
			return $this->data['milestone_rollover_month'];
		}

		return FALSE;
	}
	function setMilestoneRolloverMonth($value) {
		$value = trim($value);

		$key = Option::getByValue($value, TTDate::getMonthOfYearArray() );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->milestonerollovermonth_validator_label,
											$value,
											$this->milestonerollovermonth_validator_options_msg,
											TTDate::getMonthOfYearArray() ) ) {

			$this->data['milestone_rollover_month'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getMilestoneRolloverDayOfMonth() {
		if ( isset($this->data['milestone_rollover_day_of_month']) ) {
			return $this->data['milestone_rollover_day_of_month'];
		}

		return FALSE;
	}
	function setMilestoneRolloverDayOfMonth($value) {
		$value = trim($value);

		$key = Option::getByValue($value, TTDate::getDayOfMonthArray() );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->milestonerolloverdayofmonth_validator_label,
											$value,
											$this->milestonerolloverdayofmonth_validator_options_msg,
											TTDate::getDayOfMonthArray() ) ) {

			$this->data['milestone_rollover_day_of_month'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getMinimumEmployedDays() {
		if ( isset($this->data['minimum_employed_days']) ) {
			return (int)$this->data['minimum_employed_days'];
		}

		return FALSE;
	}
	function setMinimumEmployedDays($int) {
		$int = trim($int);

		if  ( empty($int) ){
			$int = 0;
		}

		if 	(	$this->Validator->isNumeric(		$this->minimumemployeddays_validator_label,
													$int,
													$this->minimumemployeddays_validator_isnumeric_msg) ) {
			$this->data['minimum_employed_days'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getMilesstoneRolloverDate( $user_hire_date = NULL ) {
		if ( $user_hire_date == '' ) {
			return FALSE;	
		}
		
		if ( $this->getMilestoneRolloverHireDate() == TRUE ) {
			$retval = $user_hire_date;	
		} else {
			$user_hire_date_arr = getdate( $user_hire_date );
			$retval = mktime( $user_hire_date_arr['hours'], $user_hire_date_arr['minutes'], $user_hire_date_arr['seconds'], $this->getMilestoneRolloverMonth(), $this->getMilestoneRolloverDayOfMonth(), $user_hire_date_arr['year'] );
		}
		
		Debug::Text('Milestone Rollover Date: '. TTDate::getDate('DATE+TIME', $retval) .' Hire Date: '. TTDate::getDate('DATE+TIME', $user_hire_date), __FILE__, __LINE__, __METHOD__,10);
		return $retval;	
	}
	
	function getAccrualRatePerTimeFrequency( $accrual_rate, $annual_pay_periods = NULL ) {
		$retval = FALSE;
		switch( $this->getApplyFrequency() ) {
			case 10: //Pay Period
				if ( $annual_pay_periods == '' ) {
					return FALSE;
				}
				$retval = bcdiv( $accrual_rate, $annual_pay_periods,0);
				break;
			case 20: //Year
				$retval = $accrual_rate;
			
				break;
			case 30: //Month
				$retval = bcdiv( $accrual_rate, 12,0);			
				break;
			case 40: //Week
				$retval = bcdiv( $accrual_rate, 52,0);
				break;
		}
		
		//Round to nearest minute, or 15mins?
		$retval = TTDate::roundTime( $retval, 60, 20 );
		
		Debug::Text('Accrual Rate Per Frequency: '. $retval, __FILE__, __LINE__, __METHOD__,10);
		
		return $retval;
	}
	
	function inApplyFrequencyWindow( $current_epoch, $offset, $pay_period_end_date = NULL ) {
		$retval = FALSE;
		switch( $this->getApplyFrequency() ) {
			case 10: //Pay Period
				if ( $pay_period_end_date == '' ) {
					return FALSE;
				}				
				if ( $pay_period_end_date >= ($current_epoch-$offset)
						AND $pay_period_end_date <= $current_epoch ) {
					$retval = TRUE;
				}				
				break;
			case 20: //Year
				$year_epoch = mktime( 0,0,0, $this->getApplyFrequencyMonth(), $this->getApplyFrequencyDayOfMonth(), TTDate::getYear( $current_epoch ) );
				Debug::Text('Year EPOCH: '. TTDate::getDate('DATE+TIME', $year_epoch), __FILE__, __LINE__, __METHOD__,10);
				
				if ( $year_epoch >= ($current_epoch-$offset)
						AND $year_epoch <= $current_epoch ) {
					$retval = TRUE;
				}								
				break;
			case 30: //Month
				$month_epoch = mktime( 0,0,0, TTDate::getMonth( $current_epoch ), $this->getApplyFrequencyDayOfMonth(), TTDate::getYear( $current_epoch ) );
				Debug::Text('Month EPOCH: '. TTDate::getDate('DATE+TIME', $month_epoch), __FILE__, __LINE__, __METHOD__,10);
				
				if ( $month_epoch >= ($current_epoch-$offset)
						AND $month_epoch <= $current_epoch ) {
					$retval = TRUE;
				}
				break;
			case 40: //Week
				Debug::Text('Current Day Of Week: '. TTDate::getDayOfWeek($current_epoch-$offset), __FILE__, __LINE__, __METHOD__,10);			
				if ( $this->getApplyFrequencyDayOfWeek() == TTDate::getDayOfWeek( ($current_epoch-$offset) ) ) {
					$retval = TRUE;					
				}
				break;			
		}
		
		Debug::Text('RetVal: '. (int)$retval, __FILE__, __LINE__, __METHOD__,10);
		return $retval;
	}
	
	function addAccrualPolicyTime( $epoch = NULL, $offset = 79200 ) { //22hr offset
		if ( $epoch == '' ) {
			$epoch = TTDate::getTime();
		}
		
		Debug::Text('Current EPOCH: '. TTDate::getDate('DATE+TIME', $epoch), __FILE__, __LINE__, __METHOD__,10);
						
		$pglf = new PolicyGroupListFactory();
		
		$pglf->StartTransaction();
		
		$pglf->getSearchByCompanyIdAndArrayCriteria( $this->getCompany(), array( 'accrual_policy_id' => $this->getId() ) );
		if ( $pglf->getRecordCount() > 0 ) {
			Debug::Text('Found Policy Group: ', __FILE__, __LINE__, __METHOD__,10);
			foreach( $pglf as $pg_obj ) {
				//Get all users assigned to this policy group.
				$policy_group_users = $pg_obj->getUser();
				if ( is_array($policy_group_users) AND count($policy_group_users) > 0 ) {
					Debug::Text('Found Policy Group Users: ', __FILE__, __LINE__, __METHOD__,10);
					foreach( $policy_group_users as $user_id ) {
						Debug::Text('Policy Group User ID: '. $user_id, __FILE__, __LINE__, __METHOD__,10);
						
						//Get User Object
						$ulf = new UserListFactory();
						$ulf->getByIDAndCompanyID( $user_id, $this->getCompany() );
						if ( $ulf->getRecordCount() == 1 ) {
							$u_obj = $ulf->getCurrent();
							Debug::Text('User: '. $u_obj->getFullName(), __FILE__, __LINE__, __METHOD__,10);
							if ( $this->getMinimumEmployedDays() == 0
									OR TTDate::getDays( ($epoch-$u_obj->getHireDate()) ) >= $this->getMinimumEmployedDays() ) {
								Debug::Text('&nbsp;&nbsp;User has been employed long enough.', __FILE__, __LINE__, __METHOD__,10);
								
								$annual_pay_periods = 0;
								$in_apply_frequency_window = FALSE;
								$accrual_balance = 0;
								$accrual_amount = 0;
								if ( $this->getApplyFrequency() == 10 ) {
									//Because of pay period frequencies, and users being assigned to different
									//pay period schedules we need to get the last pay period of each user individually.
									//This will return the pay period that just ended in the offset time.
									$pplf = new PayPeriodListFactory();
									$pplf->getByUserIdAndEndDate( $user_id, ($epoch-$offset) );
									if ( $pplf->getRecordCount() > 0 ) {
										foreach( $pplf as $pp_obj ) {
											Debug::Text('&nbsp;&nbsp;Pay Period End Date: '. TTDate::getDate('DATE+TIME', $pp_obj->getEndDate() ), __FILE__, __LINE__, __METHOD__,10);
											if ( $this->inApplyFrequencyWindow( $epoch, $offset, $pp_obj->getEndDate() ) == TRUE ) {
												$in_apply_frequency_window = TRUE;
												$annual_pay_periods = $pp_obj->getPayPeriodScheduleObject()->getAnnualPayPeriods();
												break;
											} else {
												Debug::Text('&nbsp;&nbsp;User not in Apply Frequency Window: ', __FILE__, __LINE__, __METHOD__,10);
											}
										}
									} else {
										Debug::Text('&nbsp;&nbsp; No Pay Period Found.', __FILE__, __LINE__, __METHOD__,10);
									}
								} elseif ( $this->inApplyFrequencyWindow( $epoch, $offset ) == TRUE ) {
									Debug::Text('&nbsp;&nbsp;User IS in NON-PayPeriod Apply Frequency Window.', __FILE__, __LINE__, __METHOD__,10);
									$in_apply_frequency_window = TRUE;
								} else {
									Debug::Text('&nbsp;&nbsp;User is not in Apply Frequency Window.', __FILE__, __LINE__, __METHOD__,10);
									$in_apply_frequency_window = FALSE;
								}
								
								if ( $in_apply_frequency_window == TRUE ) {
									$length_of_service_days = TTDate::getDays( ($epoch-$this->getMilesstoneRolloverDate( $u_obj->getHireDate() )) );
									if ( $length_of_service_days < 0 ) {
										$length_of_service_days = 0;	
									}
									Debug::Text('&nbsp;&nbsp;bUser IS in Apply Frequency Window. Length of Service Days: '. $length_of_service_days, __FILE__, __LINE__, __METHOD__,10);
									
									$apmlf = new AccrualPolicyMilestoneListFactory();									
									$apmlf->getByAccrualPolicyIdAndLengthOfServiceDays($this->getId(), $length_of_service_days );
									if ( $apmlf->getRecordCount() > 0 ) {
										$apm_obj = $apmlf->getCurrent();
										Debug::Text('&nbsp;&nbsp;Found Matching Milestone, Accrual Rate: '. $apm_obj->getAccrualRate() .'secs/year', __FILE__, __LINE__, __METHOD__,10);
																			
										//Check min/max times of accrual policy.
										$ablf = new AccrualBalanceListFactory();
										$ablf->getByUserIdAndAccrualPolicyId($user_id, $this->getId() );
										if ( $ablf->getRecordCount() > 0 ) {
											$accrual_balance = $ablf->getCurrent()->getBalance();		
										} else {
											$accrual_balance = 0;
										}
										Debug::Text('&nbsp;&nbsp; Current Accrual Balance: '. $accrual_balance, __FILE__, __LINE__, __METHOD__,10);
										
										if ( $accrual_balance < $apm_obj->getMaximumTime() ) {
													
											$accrual_amount = $this->getAccrualRatePerTimeFrequency( $apm_obj->getAccrualRate(), $annual_pay_periods );
											
											$new_accrual_balance = bcadd( $accrual_balance, $accrual_amount);
											//if ( $new_accrual_balance < $apm_obj->getMinimumTime() ) {
											//	$accrual_amount = bcsub( $apm_obj->getMinimumTime(), $accrual_balance, 0 );
											if ( $new_accrual_balance > $apm_obj->getMaximumTime() ) {
												$accrual_amount = bcsub( $apm_obj->getMaximumTime(), $accrual_balance, 0 );
											}
											Debug::Text('&nbsp;&nbsp; Min/Max Adjusted Accrual Amount: '. $accrual_amount .' Limits: Min: '. $apm_obj->getMinimumTime() .' Max: '. $apm_obj->getMaximumTime(), __FILE__, __LINE__, __METHOD__,10);
											
											//Round to nearest 1min
											$af = new AccrualFactory();
											$af->setUser( $user_id );
											$af->setType( 75 ); //Accrual Policy
											$af->setAccrualPolicyID( $this->getId() );
											$af->setAmount( $accrual_amount );
											$af->setTimeStamp( $epoch );
											$af->setEnableCalcBalance( TRUE );
									
											if ( $af->isValid() ) {
												$af->Save();
											}
											unset($accrual_amount, $accrual_balance, $new_accrual_balance);
											
													
										} else {
											Debug::Text('&nbsp;&nbsp; Accrual Balance is outside Milestone Range. Skipping...', __FILE__, __LINE__, __METHOD__,10);
													
										}
									} else {
										Debug::Text('&nbsp;&nbsp;DID NOT Find Matching Milestone.', __FILE__, __LINE__, __METHOD__,10);
									}
					
								}
							} else {
								Debug::Text('&nbsp;&nbsp;User has only been employed: '. TTDate::getDays( ($epoch-$u_obj->getHireDate()) ) .' Days, not enough.', __FILE__, __LINE__, __METHOD__,10);
							}
						} else {
							Debug::Text('No User Found. Company ID: '. $this->getCompany(), __FILE__, __LINE__, __METHOD__,10);
						}
					}
				}
			}
		}
		
		$pglf->CommitTransaction();
		
		return TRUE;
	}

	function Validate() {
		if ( $this->getDeleted() == TRUE ){
			//Check to make sure there are no hours using this OT policy.
			$alf = new AccrualListFactory();
			$alf->getByAccrualPolicyId( $this->getId() );
			if ( $alf->getRecordCount() > 0 ) {
				$this->Validator->isTRUE(	$this->inuse_validator_label,
											FALSE,
											$this->inuse_validator_msg);

			}
		}

		return TRUE;
	}

	function preSave() {
		return TRUE;
	}

	function postSave() {
		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Accrual Policy', NULL, $this->getTable() );
	}

}
?>
