<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 583 $
 * $Id: About.php 583 2007-01-29 23:59:29Z ipso $
 * $Date: 2007-01-29 15:59:29 -0800 (Mon, 29 Jan 2007) $
 */
require_once('../../includes/global.inc.php');
require_once(Environment::getBasePath() .'includes/Interface.inc.php');

//Debug::setVerbosity( 11 );

$title = 'About';
$smarty->assign('title', $title);
BreadCrumb::setCrumb($title);

/*
 * Get FORM variables
 */
extract	(FormVariables::GetVariables(
										array	(
												'action',
												) ) );

$data = $system_settings;

$action = strtolower($action);
switch ($action) {
	case 'check for updates':
		Debug::Text('Check For Update!', __FILE__, __LINE__, __METHOD__,10);
		
		$ttsc = new TimeTrexSoapClient();
		
		//We must ensure that the data is up to date
		//Otherwise version check will fail.
		$ttsc->sendCompanyData( $current_company->getId(), TRUE );
		$ttsc->sendCompanyUserLocationData( $current_company->getId() );
		$ttsc->sendCompanyUserCountData( $current_company->getId() );
		$ttsc->sendCompanyVersionData( $current_company->getId() );

		$latest_version = $ttsc->isLatestVersion( $current_company->getId() );
		$latest_tax_engine_version = $ttsc->isLatestTaxEngineVersion( $current_company->getId() );
		$latest_tax_data_version = $ttsc->isLatestTaxDataVersion( $current_company->getId() );
		
		$sslf = new SystemSettingListFactory();
		$sslf->getByName('new_version');
		if ( $sslf->getRecordCount() == 1 ) {
			$obj = $sslf->getCurrent();
		} else {
			$obj = new SystemSettingListFactory();
		}
		$obj->setName( 'new_version' );

		if( $latest_version == FALSE
				OR $latest_tax_engine_version == FALSE
				OR $latest_tax_data_version == FALSE ) {
			$obj->setValue( 1 );
			$data['new_version'] = 1;
		} else {
			$obj->setValue( 0 );
			$data['new_version'] = 0;
		}

		if ( $obj->isValid() ) {
			$obj->Save();
		}
	default:
		if ( isProfessionalEditionAvailable() == TRUE ) {
			$data['product_edition'] = 'Professional';
		} else {
			$data['product_edition'] = 'Standard';
		}
		
		
		//Get Employee counts for this month, and last month
		$month_of_year_arr = TTDate::getMonthOfYearArray();
		
		//This month
		$begin_month_epoch = TTDate::getBeginMonthEpoch();
		
		$cuclf = new CompanyUserCountListFactory();
		$cuclf->getMinAvgMaxByCompanyIdAndStartDateAndEndDate( $current_company->getId(), $begin_month_epoch, TTDate::getEndMonthEpoch($begin_month_epoch) );
		if ( $cuclf->getRecordCount() > 0 ) {
			$cuc_obj = $cuclf->getCurrent();
		
			$data['user_counts']['this_month'] = array(
															'label' => $month_of_year_arr[TTDate::getMonth( $begin_month_epoch )] .' '. TTDate::getYear($begin_month_epoch),
															'max_active_users' => $cuc_obj->getColumn('max_active_users'),
															'max_inactive_users' => $cuc_obj->getColumn('max_inactive_users'),
															'max_deleted_users' => $cuc_obj->getColumn('max_deleted_users'),
															);
		}
		
		//Last month
		$begin_last_month_epoch = TTDate::getBeginMonthEpoch($begin_month_epoch-86400);
		
		$cuclf = new CompanyUserCountListFactory();
		$cuclf->getMinAvgMaxByCompanyIdAndStartDateAndEndDate( $current_company->getId(), $begin_last_month_epoch, TTDate::getEndMonthEpoch($begin_last_month_epoch) );
		if ( $cuclf->getRecordCount() > 0 ) {
			$cuc_obj = $cuclf->getCurrent();
		
			$data['user_counts']['last_month'] = array(
															'label' => $month_of_year_arr[TTDate::getMonth( $begin_last_month_epoch )] .' '. TTDate::getYear($begin_last_month_epoch),
															'max_active_users' => $cuc_obj->getColumn('max_active_users'),
															'max_inactive_users' => $cuc_obj->getColumn('max_inactive_users'),
															'max_deleted_users' => $cuc_obj->getColumn('max_deleted_users'),
															);
		
		}
		
		$cjlf = new CronJobListFactory();
		$cjlf->getMostRecentlyRun();
		if ( $cjlf->getRecordCount() > 0 ) {
			$cj_obj = $cjlf->getCurrent();
			$data['cron'] = array(
								'last_run_date' => $cj_obj->getLastRunDate()
								);
		}
		
}

//var_dump($data);
$smarty->assign_by_ref('data', $data);

$smarty->display('help/About.tpl');
?>